==> locus-0.92.tar.gz: locus-0.92/bin/grazer/term.cpp <==
#include "myfndt.h"
#include "term.h"
#include "grazeopt.h"
#include "punctaxe.h"
#include "newcase.h"
#include "query.h"
#include "qbuild.h"
#include "softop.h"
#include "wordmask.h"
#include "frontln.h"
#include "newstem.h"
#include "locusopt.h"
#include "messform.h"

#include "osoftop.h"
#include "ooutform.h"
#include "opunct.h"
#include "ocase.h"
#include "olcase.h"
#include "oterm.h"

#include <stdio.h>

CTerm::CTerm(int Count, 
    char *Arg[], 
    FPrint Print, 
    CGrazerOptions &Options):
    Query(new CQuery),
    SoftOperator(new CSoftOperator(
	Options.COptions<double>::Get(O_FOCUSWEIGHT),
	Options.COptions<double>::Get(O_CORRELATIONWEIGHT),
	Options.COptions<double>::Get(O_PROMINENCEWEIGHT),
	Options.COptions<double>::Get(O_LOCALITYWEIGHT),
	Options.COptions<double>::Get(O_ORDERWEIGHT),
	Options.COptions<double>::Get(O_RELEVANCETHRESHOLD),
	Options.COptions<int>::Get(O_SHOWREL))),
    Mask(NULL),
    DuplWords(4, 8),
    PunctChange(0)
{
    FillQuery(Count, Arg);
    ExpandQuery(Options);
    FinalDuplCheck();
    PrintWarnings(Print, Options);
}

CTerm::CTerm(CQueryBuilder &QueryBuilder):
    Query(QueryBuilder.NewQuery()),
    SoftOperator(NULL),
    Mask(NULL),
    DuplWords(4, 8),
    PunctChange(0)
{
    Assert(Query != NULL);
    ExpandQuery(QueryBuilder.GetOptions());
    FinalDuplCheck();

    SoftOperator = new CSoftOperator(QueryBuilder.GetSoftOpName(),
        QueryBuilder.YankFilter(),
        QueryBuilder.GetOptions());
    PrintWarnings(QueryBuilder.GetPrint(), QueryBuilder.GetOptions());
}

CTerm::~CTerm()
{
    delete Mask;
    delete SoftOperator;
    delete Query;
}

bool CTerm::IsEmpty() const
{
    return !!(Query->GetWordCount() == 0);
}

int CTerm::Collate(CFrontLine &FrontLine)
{
    Assert(!Mask);
    ushort Before = FrontLine.GetFreeIndex();
    if (FillFrontLine(FrontLine, Before))
	   return 1;
    ushort After = FrontLine.GetFreeIndex();
    Assert(After > 0);
    Mask = new CWordMask(Before, After - 1);
    return 0;
}

void CTerm::AddRelevant(CResult *Result, CDocWrapper *DocWrapper)
{
    Assert(Mask != NULL);
    SoftOperator->AddRelevant(Result, DocWrapper, *Mask);
}

int CTerm::IsRelevant(CDocWrapper *DocWrapper, double &Relevance)
{
    Assert(Mask != NULL);
    return SoftOperator->IsRelevant(DocWrapper, *Mask, Relevance);
}

void CTerm::FillQuery(int Count, char *Arg[])
{
    for (int i = 1; i < Count; i++)
    {   DString ArgStr = DString(Arg[i]);
        if (!ArgStr.IsEmpty())
        {   short Dupl = Query->Add(ArgStr);
	    if (Dupl)
		AddDupl(ArgStr);
	}
    }
}

void CTerm::ExpandQuery(CGrazerOptions &Options)
{
    CPunctAxe PunctAxe(Options.Get(O_PUNCT));
    PunctChange = Query->AxePunct(PunctAxe);

    CObjPtr<CAbstractCase> CaseStandard = NewCaseStandard(
        Options.COptions<int>::Get(O_EARLYCASECONV),
        Options.COptions<int>::Get(O_LATECASECONV));
    Query->VariateCase(CaseStandard);

    // 23Nov98: grazer crashes on access violations; ElectricFence
    // makes it to crash in dlopen; not calling stemmer's
    // destructor (which calls dlopen) masks the problem
    CObjPtr<CAbstractStemmer> Stem = NewStemmer(Options);
    //CAbstractStemmer *Stem = NewStemmer(Options);
    if (Query->Stem(Stem))
	   PunctChange = 1;
}

void CTerm::FinalDuplCheck()
{
    DISArray<DString> WordList(16, 16);
    int MajorIndex = 0;
    DString IndexedWord = Query->GetWord(MajorIndex, 0);
    while (!IndexedWord.IsEmpty())
    {   if (WordList.Find(&IndexedWord) == INT_MAX)
	   {   WordList.Add(new DString(IndexedWord));
		  int MinorIndex = 1;
		  DString WordForm = Query->GetWord(MajorIndex, MinorIndex);
		  while (!WordForm.IsEmpty())
		  {   if (WordList.Find(&WordForm) == INT_MAX)
			 {   WordList.Add(new DString(WordForm));
			     ++MinorIndex;
				WordForm = Query->GetWord(MajorIndex, MinorIndex);
			 }
	           else
			 {   AddDupl(IndexedWord);
			     int KnownRet = Query->Del(MajorIndex);
				Assert(!KnownRet);
				goto CONTINUE;
			 }
		  }
	   }
	   else
	   {   AddDupl(IndexedWord);
		  int KnownRet = Query->Del(MajorIndex);
		  Assert(!KnownRet);
		  goto CONTINUE;
	   }

	   ++MajorIndex;

CONTINUE:
	   IndexedWord = Query->GetWord(MajorIndex, 0);
    }	  
}

void CTerm::AddDupl(const DString &DWord)
{
    if (DuplWords.Find(const_cast<DString *>(&DWord)) != INT_MAX)
	   return;
    DuplWords.Add(new DString(DWord));
}

int CTerm::FillFrontLine(CFrontLine &FrontLine, short Base)
{   int Broken;

    short MajorIndex = 0;
    DString IndexedWord = Query->GetWord(MajorIndex, 0);
    Assert(!IndexedWord.IsEmpty());
    do
    {   short GlobalIndex = Base + MajorIndex;

	   Broken = FrontLine.AddVariation(new CFrontItem(
		  IndexedWord, 
		  GlobalIndex));
	   if (Broken)
		  return 1;

	   int MinorIndex = 1;
	   IndexedWord = Query->GetWord(MajorIndex, MinorIndex);
	   while (!IndexedWord.IsEmpty())
	   {   Broken = FrontLine.AddVariation(new CFrontItem(
	      	 IndexedWord, 
			 GlobalIndex));
		  if (Broken)
		      return 1;

    		  ++MinorIndex;
		  IndexedWord = Query->GetWord(MajorIndex, MinorIndex);
	   }

	   ++MajorIndex;
	   IndexedWord = Query->GetWord(MajorIndex, 0);
    }   while (!IndexedWord.IsEmpty());
    
    return 0;
}
    
void CTerm::PrintWarnings(FPrint Print, CGrazerOptions &Options)
{
    DString WarnDuplWord = Options.Get(O_DUPLWORD_1);

    DISArray<DString>::DIterator Itr = DuplWords.InitIterator();
    while (DuplWords.IsDereferenceable(Itr))
    {   DString *DWord = (*Itr).p;
        CFormatString Format;
        Format << *DWord;
        DString Message = Format.GetMessage(WarnDuplWord);
    	Print(Message.c_str());
        ++Itr;
    }

    if (PunctChange)
	   DisplayWords(Print, Options);
}

void CTerm::DisplayWords(FPrint Print, CGrazerOptions &Options)
{
	Print(Options.Get(O_SOUGHT).c_str());
    DString PrimaryFormat = Options.Get(O_SOUGHTPRIMARY_1);
    DString SecondaryFormat = Options.Get(O_SOUGHTSECONDARY_1);
    DString EndVariant = Options.Get(O_ENDVARIANT);

	int i = 0;
	DString QWord = Query->GetWord(i, 0);
	while (!QWord.IsEmpty())
	{   CFormatString FormatPrimary;
        FormatPrimary << QWord;
        DString Message = FormatPrimary.GetMessage(PrimaryFormat);
    	Print(Message.c_str());

	    int j = 1;
	    QWord = Query->GetWord(i, j);
	    while (!QWord.IsEmpty())
	    {   CFormatString FormatSecondary;
            FormatSecondary << QWord;
            DString Message = FormatSecondary.GetMessage(SecondaryFormat);
            Print(Message.c_str());

		   j++;
		   QWord = Query->GetWord(i, j);
	    }

	    Print(EndVariant.c_str());

	    i++;
	    QWord = Query->GetWord(i, 0);
	}
}

==> locus-0.93.tar.gz: locus-0.93/bin/grazer/term.cpp <==
#include "myfndt.h"
#include "term.h"
#include "grazeopt.h"
#include "punctaxe.h"
#include "newcase.h"
#include "query.h"
#include "qbuild.h"
#include "softop.h"
#include "wordmask.h"
#include "frontln.h"
#include "newstem.h"
#include "locusopt.h"
#include "messform.h"

#include "osoftop.h"
#include "ooutform.h"
#include "opunct.h"
#include "ocase.h"
#include "olcase.h"
#include "oterm.h"

#include <stdio.h>

CTerm::CTerm(int Count, 
    char *Arg[], 
    FPrint Print, 
    CGrazerOptions &Options):
    Query(new CQuery),
    SoftOperator(new CSoftOperator(
	(Options.*&(COptions<double>::Get))(O_FOCUSWEIGHT),
	(Options.*&(COptions<double>::Get))(O_CORRELATIONWEIGHT),
	(Options.*&(COptions<double>::Get))(O_PROMINENCEWEIGHT),
	(Options.*&(COptions<double>::Get))(O_LOCALITYWEIGHT),
	(Options.*&(COptions<double>::Get))(O_ORDERWEIGHT),
	(Options.*&(COptions<double>::Get))(O_RELEVANCETHRESHOLD),
	(Options.*&(COptions<int>::Get))(O_SHOWREL))),
    Mask(NULL),
    DuplWords(4, 8),
    PunctChange(0)
{
    FillQuery(Count, Arg);
    ExpandQuery(Options);
    FinalDuplCheck();
    PrintWarnings(Print, Options);
}

CTerm::CTerm(CQueryBuilder &QueryBuilder):
    Query(QueryBuilder.NewQuery()),
    SoftOperator(NULL),
    Mask(NULL),
    DuplWords(4, 8),
    PunctChange(0)
{
    Assert(Query != NULL);
    ExpandQuery(QueryBuilder.GetOptions());
    FinalDuplCheck();

    SoftOperator = new CSoftOperator(QueryBuilder.GetSoftOpName(),
        QueryBuilder.YankFilter(),
        QueryBuilder.GetOptions());
    PrintWarnings(QueryBuilder.GetPrint(), QueryBuilder.GetOptions());
}

CTerm::~CTerm()
{
    delete Mask;
    delete SoftOperator;
    delete Query;
}

bool CTerm::IsEmpty() const
{
    return !!(Query->GetWordCount() == 0);
}

int CTerm::Collate(CFrontLine &FrontLine)
{
    Assert(!Mask);
    ushort Before = FrontLine.GetFreeIndex();
    if (FillFrontLine(FrontLine, Before))
	   return 1;
    ushort After = FrontLine.GetFreeIndex();
    Assert(After > 0);
    Mask = new CWordMask(Before, After - 1);
    return 0;
}

void CTerm::AddRelevant(CResult *Result, CDocWrapper *DocWrapper)
{
    Assert(Mask != NULL);
    SoftOperator->AddRelevant(Result, DocWrapper, *Mask);
}

int CTerm::IsRelevant(CDocWrapper *DocWrapper, double &Relevance)
{
    Assert(Mask != NULL);
    return SoftOperator->IsRelevant(DocWrapper, *Mask, Relevance);
}

void CTerm::FillQuery(int Count, char *Arg[])
{
    for (int i = 1; i < Count; i++)
    {   DString ArgStr = DString(Arg[i]);
        if (!ArgStr.IsEmpty())
        {   short Dupl = Query->Add(ArgStr);
	    if (Dupl)
		AddDupl(ArgStr);
	}
    }
}

void CTerm::ExpandQuery(CGrazerOptions &Options)
{
    CPunctAxe PunctAxe(Options.Get(O_PUNCT));
    PunctChange = Query->AxePunct(PunctAxe);

    CObjPtr<CAbstractCase> CaseStandard = NewCaseStandard(
        (Options.*&(COptions<int>::Get))(O_EARLYCASECONV),
        (Options.*&(COptions<int>::Get))(O_LATECASECONV));
    Query->VariateCase(CaseStandard);

    // 23Nov98: with some combinations of compiler and
    // system C library, grazer crashes on access violations;
    // ElectricFence makes it to crash in dlopen; not calling
    // stemmer's destructor (which calls dlopen) masks the
    // problem
    CObjPtr<CAbstractStemmer> Stem = NewStemmer(Options);
    if (Query->Stem(Stem))
	   PunctChange = 1;
}

void CTerm::FinalDuplCheck()
{
    DISArray<DString> WordList(16, 16);
    int MajorIndex = 0;
    DString IndexedWord = Query->GetWord(MajorIndex, 0);
    while (!IndexedWord.IsEmpty())
    {   if (WordList.Find(&IndexedWord) == INT_MAX)
	   {   WordList.Add(new DString(IndexedWord));
		  int MinorIndex = 1;
		  DString WordForm = Query->GetWord(MajorIndex, MinorIndex);
		  while (!WordForm.IsEmpty())
		  {   if (WordList.Find(&WordForm) == INT_MAX)
			 {   WordList.Add(new DString(WordForm));
			     ++MinorIndex;
				WordForm = Query->GetWord(MajorIndex, MinorIndex);
			 }
	           else
			 {   AddDupl(IndexedWord);
			     int KnownRet = Query->Del(MajorIndex);
				Assert(!KnownRet);
				goto CONTINUE;
			 }
		  }
	   }
	   else
	   {   AddDupl(IndexedWord);
		  int KnownRet = Query->Del(MajorIndex);
		  Assert(!KnownRet);
		  goto CONTINUE;
	   }

	   ++MajorIndex;

CONTINUE:
	   IndexedWord = Query->GetWord(MajorIndex, 0);
    }	  
}

void CTerm::AddDupl(const DString &DWord)
{
    if (DuplWords.Find(const_cast<DString *>(&DWord)) != INT_MAX)
	   return;
    DuplWords.Add(new DString(DWord));
}

int CTerm::FillFrontLine(CFrontLine &FrontLine, short Base)
{   int Broken;

    short MajorIndex = 0;
    DString IndexedWord = Query->GetWord(MajorIndex, 0);
    Assert(!IndexedWord.IsEmpty());
    do
    {   short GlobalIndex = Base + MajorIndex;

	   Broken = FrontLine.AddVariation(new CFrontItem(
		  IndexedWord, 
		  GlobalIndex));
	   if (Broken)
		  return 1;

	   int MinorIndex = 1;
	   IndexedWord = Query->GetWord(MajorIndex, MinorIndex);
	   while (!IndexedWord.IsEmpty())
	   {   Broken = FrontLine.AddVariation(new CFrontItem(
	      	 IndexedWord, 
			 GlobalIndex));
		  if (Broken)
		      return 1;

    		  ++MinorIndex;
		  IndexedWord = Query->GetWord(MajorIndex, MinorIndex);
	   }

	   ++MajorIndex;
	   IndexedWord = Query->GetWord(MajorIndex, 0);
    }   while (!IndexedWord.IsEmpty());
    
    return 0;
}
    
void CTerm::PrintWarnings(FPrint Print, CGrazerOptions &Options)
{
    DString WarnDuplWord = Options.Get(O_DUPLWORD_1);

    DISArray<DString>::DIterator Itr = DuplWords.InitIterator();
    while (DuplWords.IsDereferenceable(Itr))
    {   DString *DWord = (*Itr).p;
        CFormatString Format;
        Format << *DWord;
        DString Message = Format.GetMessage(WarnDuplWord);
    	Print(Message.c_str());
        ++Itr;
    }

    if (PunctChange)
	   DisplayWords(Print, Options);
}

void CTerm::DisplayWords(FPrint Print, CGrazerOptions &Options)
{
	Print(Options.Get(O_SOUGHT).c_str());
    DString PrimaryFormat = Options.Get(O_SOUGHTPRIMARY_1);
    DString SecondaryFormat = Options.Get(O_SOUGHTSECONDARY_1);
    DString EndVariant = Options.Get(O_ENDVARIANT);

	int i = 0;
	DString QWord = Query->GetWord(i, 0);
	while (!QWord.IsEmpty())
	{   CFormatString FormatPrimary;
        FormatPrimary << QWord;
        DString Message = FormatPrimary.GetMessage(PrimaryFormat);
    	Print(Message.c_str());

	    int j = 1;
	    QWord = Query->GetWord(i, j);
	    while (!QWord.IsEmpty())
	    {   CFormatString FormatSecondary;
            FormatSecondary << QWord;
            DString Message = FormatSecondary.GetMessage(SecondaryFormat);
            Print(Message.c_str());

		   j++;
		   QWord = Query->GetWord(i, j);
	    }

	    Print(EndVariant.c_str());

	    i++;
	    QWord = Query->GetWord(i, 0);
	}
}
